function [y,it]=penalta1(theta,dtheta,x0,tol,rho0,algorithm_type)
%[y,it]=penalta1(theta,dtheta,x0,tol,rho,algorithm_type)
%
% Metoda kvadraticke penalty pro reseni ulohy 
%
% min f(x) 
%  s omez. 
%  h(x)=0
%
% Tato uloha je prevedena na ulohu min theta(x), kde theta(x) = f(x) + 1/2*rho*(h(x))^2
%
%  y       ... minimum
%  it      ... pocet iteraci
%
%  theta   ... minimalizovana fce theta(x) = f(x) + 1/2*rho*(h(x))^2 
%  dtheta  ... gradient minimalizovane fce dtheta(x)= grad f(x) + rho*h'*grad h
%  x0      ... pocatecni iterace
%  tol     ... presnost reseni
%  rho0     ... velikost penalty
%  algorithm_type .. vyber metody pro minimalizaci funkce theta(x); 
%                 algorithm_type = 1 ... nelinearni sdruzene gradienty
%                 algorithm_type = 2 ... metoda nejvetsiho spadu


global rho
rho=rho0;

switch algorithm_type
    case 1, %1. nelinearni sdruzene gradienty
        [y,it]=nonlinCG_gl(theta,dtheta,x0,tol,2);
    case 2, %2. metoda nejvetsiho spadu
        [y,it]=gradient3(theta,dtheta,x0,tol);
end;

