function [y,it]=nonlinCG_gl(f,df,x0,tol,betatype)
%[y,it]=nonlinCG_gl(f,df,x0,tol,betatype)
%
% Metoda nelinearnich sdruzenych gradientu pro minimalizaci fce f(x)
%
%  y       ... minimum
%  it      ... pocet iteraci
%
%  f       ... minimalizovana fce f(x) 
%  df	   ... gradient minimalizovane fce f(x)
%  x0      ... pocatecni iterace
%  tol     ... presnost reseni
%  betatype .. vyber vzorce pro vypocet beta; 
%                 betatype = 1 ... Fletcher-Reeves
%                 betatype = 2 ... Polak-Ribiere

koef=0.9;
alpha=0.1;

it=1;
x=x0;

grad=feval(df,x);
p=grad;


% kontrola smeru p
if p'*grad<0
    p=grad;
end;

p=p/norm(p);

% line search
alfa=zlatyrez(f,0,100,x,-p,100*tol);

%x=x-alfa*p;
h=-alfa*p;

%Omezeni dlouhych kroku
lambda=1;
while feval(f,x+lambda*h) > feval(f,x)+alpha*lambda*grad'*h
    lambda=lambda*koef;
end;

xn=x+lambda*h;

grad_n=feval(df,xn);

% vypocet beta
switch betatype
    case 1, %1. Fletcher-Reeves
        beta=-(grad_n'*grad_n)/(grad'*grad);
    case 2, %2. Polak-Ribiere
        beta=-(grad_n'*(grad_n-grad))/(grad'*grad);
end;

% sestrojeni konjugovaneho smeru
p=grad_n-beta*p;

while norm(xn-x)>tol  %norm(fxn-fx) >= tol %norm(grad_n) >= tol
  it=it+1;
  x = xn;
  grad=grad_n;
  
  % kontrola smeru p
  if p'*grad<0
     p=grad;
  end;
  
  p=p/norm(p);
  
  % line search
  alfa=zlatyrez(f,0,100,x,-p,100*tol);
  
  %x=x-alfa*p;
  h=-alfa*p;
  
  %Omezeni dlouhych kroku
  lambda=1;
  while feval(f,x+lambda*h) > feval(f,x)+alpha*lambda*grad'*h
      lambda=lambda*koef;
  end;

  xn=x+lambda*h;
  
  grad_n=feval(df,xn);
  
  % vypocet beta
  switch betatype
    case 1, %1. Fletcher-Reeves
      beta=-(grad_n'*grad_n)/(grad'*grad);
    case 2, %2. Polak-Ribiere
      beta=-(grad_n'*(grad_n-grad))/(grad'*grad);
  end;      
  
  % sestrojeni konjugovaneho smeru
  p=grad_n-beta*p;
  
end;

y=x;

% end