function [y,it_out]=penalta2(theta,dtheta,x0,tol,rho0,max_it,algorithm_type)
%[y,it_out]=penalta2(theta,dtheta,x0,tol,rho0,max_it,algorithm_type)
%
% Sekvencni metoda kvadraticke penalty pro reseni ulohy 
%
% min f(x) 
%  s omez. 
%  h(x)<=0
%
% Tato uloha je prevedena na sekvenci uloh min theta(x), 
% kde theta(x) = f(x) + 1/2*rho_n*(max{0,h(x)})^2
%
%  y       ... minimum
%  it_out  ... pocet vnejsich iteraci
%
%  theta   ... minimalizovana fce theta(x) = f(x) + 1/2*rho*(max{0,h(x)})^2 
%  dtheta  ... gradient minimalizovane fce dtheta(x)= 
%                  = grad f(x) + rho*(max{0,h(x)})'*grad (max{0,h(x)})
%  x0      ... pocatecni iterace
%  tol     ... presnost reseni
%  rho0    ... pocatecni velikost penalty
%  max_it  ... maximalni pocet vnejsich iteraci 
%  algorithm_type .. vyber metody pro minimalizaci funkce theta(x); 
%                 algorithm_type = 1 ... nelinearni sdruzene gradienty
%                 algorithm_type = 2 ... metoda nejvetsiho spadu

if tol<1e-1
    epsil=1e-1;
else
    epsil=100*tol;
end

x=x0;

global rho
rho=rho0;

alpha=10;

it_out=1;

while it_out<max_it
    it_out=it_out+1;

    switch algorithm_type
        case 1, %1. nelinearni sdruzene gradienty
            [x,it]=nonlinCG_gl(theta,dtheta,x,epsil,2);
        case 2, %2. metoda nejvetsiho spadu
            [x,it]=gradient3(theta,dtheta,x,epsil);
    end;
        
    if epsil<=tol
        y=x;
        return
    else
        rho=alpha*rho;
        epsil=1/alpha*epsil;
    end
end

y=x;
