%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Implementace algoritmu MCRS v MATLABu %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function mcrs(n, validate, fitness, niter, eps_f)
% n - pocet promennych minimalizovane funkce 
% validate - nazev funkce, ktera zajisti, ze prvek bude patrit do vyhledavaciho prostoru
% fitness - nazev funkce, kterou chceme minimalizovat
% niter - max. pocet vyhodnoceni cenove funkce
% eps_F - typicka absolutni presnost funkcnich hodnot
%
% Hledani je ukonceno:
%   a) rozsah funkcnich hodnot prvku v prac. mnozine je mensi nez eps_f
%   b) pocet iteraci (vyhodn. cenove funkce) prekroci niter


% x(1),..., x(n) \in <C_0,C_1>
  C_0=-1e4;
  C_1=1e4;

% 1. Nastaveni pouzitych promennych
  format short e

  % Parametry alg. MCRS
  %m=n*n;   % definovani velikosti pracovni mnoziny
  m=10*n; % definovani velikosti pracovni mnoziny (vetsinou postacujici)
  alfa=4;     %  definovani rozsahu preklopeni
  if ~exist('niter','var'), niter=1e+3; end;
  if ~exist('eps_f','var'), eps_f=1.0e-10; end;
  buf=[]; % Uchovani prubehu opt. procesu
  StartTime=cputime; % Pocatecni cas

  % Pracovni mnozina
  P=zeros(m,n);  % Pracovni mnozina slozena z prvku vyhledavaciho prostoru
  f=zeros(m,1);  % Odpovidajici funkcni hodnoty prvku z pracovni mnoziny
  pk=zeros(m,1); % Permutace vzestupneho serazeni 

  % Simplex
  is=zeros(n,1); % Indexy vybrane n-tice
  xt=zeros(n,1); % Teziste vybrane n-tice
  ip=0;          % Index preklapeneho bodu

  % Novy bod
  xn=zeros(n,1); % Poloha noveho bodu po preklopeni
  fn=0;          % Odpovidajici funkcni hodnota

  % Pomocne
  vi=zeros(m,1);  % Mnozina vybiranych indexu
  w=zeros(n,1);   % Pomocny vektor

  % Nastaveni citace iteraci
  iter=0; % Pocet vsech iteraci
  adm_iter=0; % Pocet pripustnych iteraci

  % Inicializace nahodneho generatoru
  rand('state',sum(100*clock));

  % Pocatecni obsazeni pracovni mnoziny
  k=1;
  while k<=m
    w=(C_1-C_0).*rand(n,1)+C_0.*ones(n,1);
    f(k)=feval(fitness,w);
    P(k,:)=w';
    [k, cputime-StartTime, f(k), P(k,:)]
    k=k+1;
  end;
  [ans,pk]=sort(f,1); % Serazeni pracovni mnoziny vzestupne podle funkcnich hodnot
  iter=m;
  adm_iter=m;


  % Postupne zlepsovani pracovni mnoziny
  while f(pk(m))-f(pk(1))>eps_f
    iter=iter+1;

    % Vyber simplexu  ==  n-tice bodu pro teziste  a  jednoho pro preklopeni
    vi=(1:m)';
    for k=1:n
      ni=floor((m-k+1)*rand+1);
      is(k)=vi(ni);
      vi(ni)=vi(m-k+1);
    end;
    ni=floor((m-n)*rand+1);
    ip=vi(ni);

    % Preklopeni simplexu
    xt=sum(P(is,:)',2)/n;
  	Gama=alfa*rand;
    xn=xt+Gama*( xt - P(ip,:)' );


    % Test pripustnosti noveho bodu
    xn=feval(validate,xn,n,C_0,C_1);

    fn=feval(fitness, xn);
    adm_iter=adm_iter+1;

  	% Zarazeni bodu do pracovni mnoziny na patricne misto
    if fn<f(pk(m))
       kam=1;
       for k=fliplr(1:m-1)
         if fn>=f(pk(k)),  kam=k+1; break;
         else
	        pom=pk(k);
    	    pk(k)=pk(k+1);
	        pk(k+1)=pom;
         end;
       end;

       P(pk(kam),:)=xn';
	   f(pk(kam))=fn;
    end;

  	% Tisk doposud nejlepsiho nalezeneho kandidata na obrazovku po kazdych 100 vyhodnocenich
    %   cenove funkce
    if mod(adm_iter,100) == 0
       [adm_iter, cputime-StartTime, f(pk(1)), P(pk(1),:)]
   	   buf=[buf; [adm_iter, cputime-StartTime, f(pk(1)), P(pk(1),:)]];
    end;
    if mod(adm_iter,100) == 0
       save vysl.txt buf -ascii
    end;


   	% Ukonceni algoritmu MCRS po niter-tem vyhodnoceni cenove funkce 
    if adm_iter>niter, break; end;
  end;  
  
  save vysl.txt buf -ascii
