function [y,it] = cg(A,b,x0,tol)

%[y,it] = cg(A,b,x0,tol)
%
% The routine finds a minimum of a function f(x)=0.5*x'*A*x-b'*x by
% linear conjugate gradient method
%
%  y   ... minimum
%  it  ... number of iterations
%
%  A ... positive definite matrix
%  b ... vector b
%  x0  ... starting point
%  tol ... stopping criterion

 n=max(size(A));
 
 it=0;
 x = x0;
 r = A*x-b;
 p = r;
 
 while norm(r)>tol
   it = it + 1;
   alpha = (r'*p)/(p'*A*p);
   x = x - alpha*p;
   r = r - alpha*A*p;
   beta=(r'*A*p)/(p'*A*p); 
   p = r - beta*p;
 end

 y=x;

%end
