function [y,it]=nonlinCG(f,df,x0,tol,betatype)

%[y,it]=nonlinCG(f,df,x0,tol,betatype)
%
% Metoda nelinearnich sdruzenych gradientu pro minimalizaci fce f(x)
%
%  y       ... minimum
%  it      ... pocet iteraci
%
%  f       ... minimalizovana fce f(x) 
%  df	   ... gradient minimalizovane fce f(x)
%  x0      ... pocatecni iterace
%  tol     ... presnost reseni
%  betatype .. vyber vzorce pro vypocet beta; 
%                 betatype = 1 ... Fletcher-Reeves
%                 betatype = 2 ... Polak-Ribiere


it=0;
x=x0;

grad_n=feval(df,x);
p=grad_n;

while norm(grad_n)>tol  %norm(fxn-fx) >= tol %norm(xn-x) >= tol
  it=it+1;
  grad=grad_n;
  
  % kontrola smeru p
  if p'*grad<0
     p=grad;
  end;
  
  p=p/norm(p);
  
  % line search
  alfa=zlatyrez(f,0,100,x,-p,tol);
  
  x=x-alfa*p;
  
  grad_n=feval(df,x);
  
  % vypocet beta
  switch betatype
    case 1, %1. Fletcher-Reeves
      beta=-(grad_n'*grad_n)/(grad'*grad);
    case 2, %2. Polak-Ribiere
      beta=-(grad_n'*(grad_n-grad))/(grad'*grad);
  end;      
  
  % sestrojeni konjugovaneho smeru
  p=grad_n-beta*p;
  
end;

y=x;

% end