function [t,it]=zlatyrez(fun,a,b,x,ddd,tol)

%[x,it]=zlatyrez(fun,a,b,x,ddd,tol)
%
% funkce nalezne minimum funkce f(x+t*ddd), t>0
% na intervalu <a,b> metodou zlateho rezu
% (realizace 'line search' v bode x ve smeru ddd)
%
% x ... minimum
% it ... pocet iteraci
%
% fun ... f(x)
% a,b ... meze intervalu
% x   ... aktualni hodnota x
% ddd   ... smer poklesu
% tol ... pozadovana absolutni presnost


koef=1/2 + sqrt(5)/2;

a0=a; b0=b;

d0=(b0-a0)/koef + a0;
c0=a0+b0-d0;

it=0;

an=a0;
bn=b0;
cn=c0;
dn=d0;

t=(cn+dn)/2;

fcn=f_t(fun,cn,x,ddd);
fdn=f_t(fun,dn,x,ddd);



while bn-an > tol
   a=an; b=bn; 
   c=cn; d=dn;
   
   fc=fcn; fd=fdn;
   
   %Pokud se poloha d lisi od spravne polohy d je nutno
   %znovu vypocist c a d 
   if abs(d - ((b-a)/koef + a))>tol/10
      d=(b-a)/koef + a;
      c=a+b-d;
      
      fc=f_t(fun,c,x,ddd);
      fd=f_t(fun,d,x,ddd);
   end;
   
   if fc < fd
      an=a; bn=d; dn=c; cn=an+bn-dn; t=dn;
      fcn=f_t(fun,cn,x,ddd); fdn=fc;
   elseif fd < fc	
      an=c; bn=b; cn=d; dn=an+bn-cn; t=cn;
      fcn=fd; fdn=f_t(fun,dn,x,ddd);
   elseif fc == fd
      an=c; bn=d; dn=(bn-an)/koef + an; cn=an+bn-dn;
      fcn=f_t(fun,cn,x,ddd); fdn=f_t(fun,dn,x,ddd);
      if fcn < fdn
         t=cn;
      else
         t=dn;
      end;
   end;
   
   
   it=it+1;
end;

