function [y,it]=trustreg(f,df,ddf,x0,delta0,tol)

% [y,it]=trustreg(f,df,ddf,x0,delta0,tol)
%
% Trust region metoda pro minimalizaci fce f(x)
%
%  y   .... minimum
%  it  .... pocet iteraci
%
%  f   .... minimalizovana fce f(x) zadana ve stringu 
%  df  .... gradient minimalizovane fce f(x) zadana ve stringu 
%  ddf .... Hessian minimalizovane fce f(x) zadana ve stringu 
%  x0  .... pocatecni iterace
%  delta0 . pocatecni velikost oblasti, na ktere hledame bod dalsi iterace
%  tol .... presnost reseni

delta=delta0;
it=0;
x=x0;

grad=feval(df,x);
Hess=feval(ddf,x);
fun_n=feval(f,x);

while norm(grad)>tol %norm(fxn-fx) >= tol %norm(xn-x) >= tol
  it=it+1;
  fun=fun_n;

  % Newtonuv krok
  h=-Hess\grad;

  if norm(h)>delta

    % Hledame mi tak, aby ||h(mi)||=delta
    konec=0;
    a=0; b=1e8;
    ha=h; hb=-(Hess+b*eye(size(Hess)))\grad;

    while ~konec
      if (norm(ha)-delta)*(norm(hb)-delta) <= 0
        mi=bisekce(grad,Hess,delta,a,b,tol);
        konec=1;
      else
        a=b; b=b+1e8;
        ha=hb; hb=-(Hess+b*eye(size(Hess)))\grad;
      end;

    end;
   
    % Krok lezici v oblasti "duvery"
    h=-(Hess+mi*eye(size(Hess)))\grad;
  end;

  % model funkce f v bode x+h
  m_c = fun + grad'*h + 1/2*h'*Hess*h;

  x=x+h;

  grad=feval(df,x);
  Hess=feval(ddf,x);
  fun_n=feval(f,x);

  % Uprava velikosti oblasti, na ktere ocekavame "dobrou" shodu 
  % fce s jejim modelem
  if ( fun_n - fun ) <= 0.75*( m_c - fun )
    delta=2*delta;
  elseif ( fun_n - fun ) >= 0.1*( m_c - fun )
    delta=1/2*delta;
  end;
end;

y=x;

% end