function [y,it]=bfgs(df,x0,tol)

%[y,it]=bfgs(df,x0,tol)
%
% Quasi-Newtonova metoda pro minimalizaci fce f(x)
% s pouzitim aproximace Hessovy matice pomoci metody BFGS 
% (Broyden, Fletcher, Goldfarb, Shanno)
%
%  y   ... minimum
%  it  ... pocet iteraci
%
%  df	... gradient minimalizovane fce f(x) zadana ve stringu 
%  x0  ... pocatecni iterace
%  tol ... presnost reseni

it=0;
xn=x0;
grad_n=feval(df,xn);

% Hess_0=I
Hess_n=eye(length(xn),length(xn));

while norm(grad_n)>tol %norm(fxn-fx) >= tol %norm(xn-x) >= tol  
  x=xn; grad=grad_n; Hess=Hess_n;

  it=it+1;
  h=-Hess\grad;
  xn=x+h;

  grad_n=feval(df,xn);

  %Konstrukce Hess_n
  yc=grad_n - grad;
  sc=xn-x;

  if yc'*sc < eps      
      Hess_n=Hess
  else
      Hess_n=Hess + (yc*yc')/(yc'*sc) - (Hess*sc*sc'*Hess)/(sc'*Hess*sc)
  end
end;

y=xn;

% end